window.Gleam.initSearch([{"doc":"treelist","title":"treelist","content":"# treelist\n\nTreelist implementation from glib (https://github.com/pendletong/glib) as a standalone module\nThis implements an AVL tree to allow for a very fast list manipulation which performs best for larger lists\nwhere items are inserted rather than prepended\n\n- Benchmarks show around 3-6 times faster than a non-prepend addition to a standard gleam list with 1000 items\n- Arbitrary removal from a treelist becomes around 6 times faster than a standard list around 1000 items and this increases to about 40 times faster at 10000 items\n- Replacing arbitrary elements in a treelist is about 1.4 times faster with 100 items up to over 100 times faster for 10000 items.\n- Memory usage of a treelist appears to be reasonable and if the list is being constantly modified/added to/removed arbitrarily\nthen usage seems to be lighter than for standard lists.\n\nIf the ability to access specific elements in a large array or removal of said elements is absolutely necessary and\nthis needs to work in both erlang and javascript targets then this is a good option.\n\n[![Package Version](https://img.shields.io/hexpm/v/treelist)](https://hex.pm/packages/treelist)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/treelist/)\n\n```sh\ngleam add treelist@1\n```\n```gleam\nimport treelist\n\npub fn main() {\n  let list = treelist.new()\n  let assert Ok(new_list) = treelist.add(list, &quot;Test&quot;)\n  treelist.get(new_list, 0)\n  // -&gt; Ok(&quot;Test&quot;)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/treelist&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"treelist","title":"Node","content":"pub opaque type Node(value)\n\n\nSynonyms:\ntreelist.Node\ntreelist Node","url":"treelist.html#Node"},{"doc":"treelist","title":"TreeList","content":"pub opaque type TreeList(value)\n\n\nSynonyms:\ntreelist.TreeList\ntreelist TreeList","url":"treelist.html#TreeList"},{"doc":"treelist","title":"add","content":"pub fn add(\n  list: TreeList(a),\n  value: a,\n) -&gt; Result(TreeList(a), Nil)\n Adds an element to the end of the provided treelist\n i.e. insert at position size(list)\n \n Returns a new TreeList containing the provided element\n \n \n let list = new()\n let assert Ok(new_list) = add(list, &quot;Test&quot;)\n get(new_list, 0)\n // -&gt; Ok(&quot;Test&quot;)\n \n \nSynonyms:\ntreelist.add\ntreelist add","url":"treelist.html#add"},{"doc":"treelist","title":"append","content":"pub fn append(\n  tlist: TreeList(a),\n  tlist2: TreeList(a),\n) -&gt; Result(TreeList(a), Nil)\n Joins one treelist onto the end of another.\n\n ## Examples\n\n \n let assert Ok(l1) = treelist.from_list([1, 2])\n let assert Ok(l2) = treelist.from_list([3])\n let assert Ok(list) = append(l1, l2)\n to_list(list)\n // -&gt; [1, 2, 3]\n \n\nSynonyms:\ntreelist.append\ntreelist append","url":"treelist.html#append"},{"doc":"treelist","title":"contains","content":"pub fn contains(tlist: TreeList(a), item: a) -&gt; Bool\n Returns true if this list contains the specified element.\n \n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n contains(list, 3)\n // -&gt; True\n \n \n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n contains(list, 999)\n // -&gt; False\n \n \nSynonyms:\ntreelist.contains\ntreelist contains","url":"treelist.html#contains"},{"doc":"treelist","title":"do_reverse","content":"pub fn do_reverse(node: Node(a)) -&gt; Node(a)\n\nSynonyms:\ntreelist.do_reverse\ntreelist do_reverse","url":"treelist.html#do_reverse"},{"doc":"treelist","title":"drop","content":"pub fn drop(tlist: TreeList(a), up_to_n: Int) -&gt; TreeList(a)\n Returns a treelist that is the given treelist with up to the given number of\n elements removed from the front of the treelist.\n\n If the treelist has less than the number of elements an empty treelist is\n returned.\n\n ## Examples\n\n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n drop(list, 2)\n |&gt; to_list\n // -&gt; [3, 4]\n \n\n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n drop(list, 9)\n |&gt; to_list\n // -&gt; []\n \n\nSynonyms:\ntreelist.drop\ntreelist drop","url":"treelist.html#drop"},{"doc":"treelist","title":"filter","content":"pub fn filter(\n  tlist: TreeList(a),\n  filter_fn: fn(a) -&gt; Bool,\n) -&gt; TreeList(a)\n Returns a new treelist containing only the elements from the first treelist for\n which the given functions returns `True`.\n\n ## Examples\n\n \n let assert Ok(list) = from_list([2, 4, 6, 1])\n filter(list), fn(x) { x &gt; 2 })\n |&gt; to_list\n // -&gt; [4, 6]\n \n\n \n let assert Ok(list) = from_list([2, 4, 6, 1])\n filter(list), fn(x) { x &gt; 6 })\n |&gt; to_list\n // -&gt; []\n \n\nSynonyms:\ntreelist.filter\ntreelist filter","url":"treelist.html#filter"},{"doc":"treelist","title":"filter_map","content":"pub fn filter_map(\n  tlist: TreeList(a),\n  filter_fn: fn(a) -&gt; Result(b, c),\n) -&gt; TreeList(b)\n Returns a new treelist containing only the elements from the first treelist for\n which the given functions returns `Ok(_)`.\n\n ## Examples\n\n \n let assert Ok(list) = from_list([2, 4, 6, 1])\n filter_map(list, Error)\n // -&gt; []\n \n\n \n let assert Ok(list) = from_list([2, 4, 6, 1])\n filter_map(list, fn(x) { Ok(x + 1) })\n |&gt; to_list\n // -&gt; [3, 5, 7, 2]\n \n\nSynonyms:\ntreelist.filter_map\ntreelist filter_map","url":"treelist.html#filter_map"},{"doc":"treelist","title":"first","content":"pub fn first(tlist: TreeList(a)) -&gt; Result(a, Nil)\n Gets the first element from the start of the treelist, if there is one.\n\n ## Examples\n\n \n first(new())\n // -&gt; Error(Nil)\n \n\n \n let assert Ok(list) = from_list([0])\n first(list)\n // -&gt; Ok(0)\n \n\n \n let assert Ok(list) = from_list([1, 2])\n first(list)\n // -&gt; Ok(1)\n \n\nSynonyms:\ntreelist.first\ntreelist first","url":"treelist.html#first"},{"doc":"treelist","title":"fold","content":"pub fn fold(\n  over list: TreeList(a),\n  from initial: b,\n  with fun: fn(b, a) -&gt; b,\n) -&gt; b\n Reduces a treelist of elements into a single value by calling a given function\n on each element, going from left to right.\n\n `fold([1, 2, 3], 0, add)` is the equivalent of\n `add(add(add(0, 1), 2), 3)`.\n\nSynonyms:\ntreelist.fold\ntreelist fold","url":"treelist.html#fold"},{"doc":"treelist","title":"fold_right","content":"pub fn fold_right(\n  over list: TreeList(a),\n  from initial: b,\n  with fun: fn(b, a) -&gt; b,\n) -&gt; b\n Reduces a treelist of elements into a single value by calling a given function\n on each element, going from right to left.\n\n `fold_right([1, 2, 3], 0, add)` is the equivalent of\n `add(add(add(0, 3), 2), 1)`.\n \nSynonyms:\ntreelist.fold_right\ntreelist fold_right","url":"treelist.html#fold_right"},{"doc":"treelist","title":"from_list","content":"pub fn from_list(list: List(a)) -&gt; Result(TreeList(a), Nil)\n Takes a list and returns a new TreeList containing all the\n elements from the list in the same order as that list\n \n Returns an Error(Nil) in the case that the list is too large\n \n \n let assert Ok(list) = from_list([1,2,3])\n get(list, 1)\n // -&gt; Ok(2)\n \n \nSynonyms:\ntreelist.from_list\ntreelist from_list","url":"treelist.html#from_list"},{"doc":"treelist","title":"get","content":"pub fn get(list: TreeList(a), index: Int) -&gt; Result(a, Nil)\n Returns the element at the specified position in the provided treelist\n \n Returns an Error(Nil) if the index is outside the allowed range\n \n Index is zero based\n \n \n ## Examples\n \n \n let list = new()\n let assert Ok(new_list) = add(list, &quot;Test&quot;)\n get(new_list, 0)\n // -&gt; Ok(&quot;Test&quot;)\n \n \n \n new() |&gt; get(0)\n // -&gt; Error(Nil)\n \n \nSynonyms:\ntreelist.get\ntreelist get","url":"treelist.html#get"},{"doc":"treelist","title":"index_of","content":"pub fn index_of(tlist: TreeList(a), item: a) -&gt; Int\n Returns the index of the first occurrence of the specified element\n in this list, or -1 if this list does not contain the element.\n \n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n index_of(list, 3)\n // -&gt; 2\n \n \n \n let assert Ok(list) = from_list([1, 2, 3, 4, 2, 2])\n index_of(list, 2)\n // -&gt; 1\n \n \n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n index_of(list, 999)\n // -&gt; -1\n \n\nSynonyms:\ntreelist.index_of\ntreelist index_of","url":"treelist.html#index_of"},{"doc":"treelist","title":"insert","content":"pub fn insert(\n  list: TreeList(a),\n  index: Int,\n  value: a,\n) -&gt; Result(TreeList(a), Nil)\n Inserts an element at the specified index in the provided treelist\n \n Returns an Error(Nil) if the index is outside the allowed range\n \n Index is zero based\n \n Returns a new TreeList containing the provided element\n \n \n let list = new()\n let assert Ok(new_list) = insert(list, 0, &quot;Test&quot;)\n get(new_list, 0)\n // -&gt; Ok(&quot;Test&quot;)\n \n \n \n let list = new()\n insert(list, 1, &quot;Test&quot;)\n // -&gt; Error(Nil)\n \n \nSynonyms:\ntreelist.insert\ntreelist insert","url":"treelist.html#insert"},{"doc":"treelist","title":"last","content":"pub fn last(tlist: TreeList(a)) -&gt; Result(a, Nil)\n Gets the last element from the start of the treelist, if there is one.\n\n ## Examples\n\n \n last(new())\n // -&gt; Error(Nil)\n \n\n \n let assert Ok(list) = from_list([0])\n last(list)\n // -&gt; Ok(0)\n \n\n \n let assert Ok(list) = from_list([1, 2])\n last(list)\n // -&gt; Ok(2)\n \n\nSynonyms:\ntreelist.last\ntreelist last","url":"treelist.html#last"},{"doc":"treelist","title":"last_index_of","content":"pub fn last_index_of(tlist: TreeList(a), item: a) -&gt; Int\n Returns the index of the last occurrence of the specified element\n in this list, or -1 if this list does not contain the element.\n \n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n last_index_of(list, 3)\n // -&gt; 2\n \n \n \n let assert Ok(list) = from_list([1, 2, 3, 4, 2, 2])\n last_index_of(list, 2)\n // -&gt; 5\n \n \n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n last_index_of(list, 999)\n // -&gt; -1\n \n\nSynonyms:\ntreelist.last_index_of\ntreelist last_index_of","url":"treelist.html#last_index_of"},{"doc":"treelist","title":"map","content":"pub fn map(\n  tlist: TreeList(a),\n  filter_fn: fn(a) -&gt; b,\n) -&gt; TreeList(b)\n Returns a new treelist containing only the elements of the first list after \n the function has been applied to each one.\n\n ## Examples\n\n \n let assert Ok(list) = from_list([2, 4, 6])\n map(list, fn(x) { x * 2 })\n |&gt; to_list\n // -&gt; [4, 8, 12]\n \n\nSynonyms:\ntreelist.map\ntreelist map","url":"treelist.html#map"},{"doc":"treelist","title":"new","content":"pub fn new() -&gt; TreeList(a)\n Creates an empty treelist\nSynonyms:\ntreelist.new\ntreelist new","url":"treelist.html#new"},{"doc":"treelist","title":"remove","content":"pub fn remove(\n  list: TreeList(a),\n  index: Int,\n) -&gt; Result(#(a, TreeList(a)), Nil)\n Removes an element at the specified index in the provided treelist\n \n Returns an Error(Nil) if the index is outside the allowed range\n \n Index is zero based\n \n Returns a tuple containing the value at the specified index and the new TreeList\n \n \n let list = new()\n let assert Ok(new_list) = insert(list, 0, &quot;Test&quot;)\n get(new_list, 0)\n // -&gt; Ok(&quot;Test&quot;)\n remove(new_list, 0)\n // -&gt; #(&quot;Test&quot;, TreeList(..))\n \n \n \n let list = new()\n remove(list, 1)\n // -&gt; Error(Nil)\n \n \nSynonyms:\ntreelist.remove\ntreelist remove","url":"treelist.html#remove"},{"doc":"treelist","title":"repeat","content":"pub fn repeat(\n  item a: a,\n  times times: Int,\n) -&gt; Result(TreeList(a), Nil)\n Builds a list of a given value a given number of times.\n\n ## Examples\n\n \n repeat(&quot;a&quot;, times: 0)\n // -&gt; new()\n \n\n \n repeat(&quot;a&quot;, times: 5)\n |&gt; to_list\n // -&gt; [&quot;a&quot;, &quot;a&quot;, &quot;a&quot;, &quot;a&quot;, &quot;a&quot;]\n \n\nSynonyms:\ntreelist.repeat\ntreelist repeat","url":"treelist.html#repeat"},{"doc":"treelist","title":"rest","content":"pub fn rest(tlist: TreeList(a)) -&gt; Result(TreeList(a), Nil)\n Returns a new treelist minus the first element. If the treelist is empty, \n `Error(Nil)` is returned.\n\n\n ## Examples\n\n \n rest(new())\n // -&gt; Error(Nil)\n \n\n \n let assert Ok(list) = from_list([0])\n rest(list)\n // -&gt; Ok([])\n \n\n \n let assert Ok(list) = from_list([1, 2])\n rest(list)\n // -&gt; Ok([2])\n \n\nSynonyms:\ntreelist.rest\ntreelist rest","url":"treelist.html#rest"},{"doc":"treelist","title":"reverse","content":"pub fn reverse(tlist: TreeList(a)) -&gt; TreeList(a)\n Creates a new treelist from a given treelist containing the same elements but in the\n opposite order.\n\n ## Examples\n\n \n reverse(new())\n |&gt; to_list\n // -&gt; []\n \n\n \n let assert Ok(list) = from_list([1])\n reverse(list)\n |&gt; to_list\n // -&gt; [1]\n \n\n \n let assert Ok(list) = from_list([1, 2])\n reverse(list)\n |&gt; to_list\n // -&gt; [2, 1]\n \n\nSynonyms:\ntreelist.reverse\ntreelist reverse","url":"treelist.html#reverse"},{"doc":"treelist","title":"set","content":"pub fn set(\n  list: TreeList(a),\n  index: Int,\n  value: a,\n) -&gt; Result(TreeList(a), Nil)\n Updates the element at index specified in the provided treelist\n \n Returns an Error(Nil) if the index is outside the allowed range\n \n Index is zero based\n \n Returns a new TreeList containing the updated node\n \n \n let list = new()\n let assert Ok(new_list) = add(list, &quot;Test&quot;)\n get(new_list, 0)\n // -&gt; Ok(&quot;Test&quot;)\n let assert Ok(new_list) = set(list, 0, &quot;Updated&quot;)\n get(new_list, 0)\n // -&gt; Ok(&quot;Updated&quot;)\n \n \nSynonyms:\ntreelist.set\ntreelist set","url":"treelist.html#set"},{"doc":"treelist","title":"size","content":"pub fn size(list: TreeList(a)) -&gt; Int\n Returns the number of elements in the provided treelist\nSynonyms:\ntreelist.size\ntreelist size","url":"treelist.html#size"},{"doc":"treelist","title":"take","content":"pub fn take(tlist: TreeList(a), up_to_n: Int) -&gt; TreeList(a)\n Returns a treelist containing the first given number of elements from the given\n treelist.\n\n If the treelist has less than the number of elements then the full treelist is\n returned.\n\n\n ## Examples\n\n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n take(list, 2)\n |&gt; to_list\n // -&gt; [1, 2]\n \n\n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n take(list, 9)\n |&gt; to_list\n // -&gt; [1, 2, 3, 4]\n \n\nSynonyms:\ntreelist.take\ntreelist take","url":"treelist.html#take"},{"doc":"treelist","title":"to_iterator","content":"pub fn to_iterator(tlist: TreeList(a)) -&gt; Iterator(a)\n Creates an iterator that yields each element from the given treelist.\n\n\n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n to_iterator(list)\n |&gt; to_list\n // -&gt; [1, 2, 3, 4]\n \n\nSynonyms:\ntreelist.to_iterator\ntreelist to_iterator","url":"treelist.html#to_iterator"},{"doc":"treelist","title":"to_iterator_reverse","content":"pub fn to_iterator_reverse(tlist: TreeList(a)) -&gt; Iterator(a)\n Creates an iterator that yields each element from the given treelist.\n\n\n \n let assert Ok(list) = from_list([1, 2, 3, 4])\n to_iterator_reverse(list)\n |&gt; to_list\n // -&gt; [4, 3, 2, 1]\n \n\nSynonyms:\ntreelist.to_iterator_reverse\ntreelist to_iterator_reverse","url":"treelist.html#to_iterator_reverse"},{"doc":"treelist","title":"to_list","content":"pub fn to_list(l: TreeList(a)) -&gt; List(a)\n Converts a TreeList into a standard Gleam list\n \n \n let list = new()\n let assert Ok(new_list) = insert(list, 0, &quot;Test&quot;)\n let assert Ok(new_list2) = insert(new_list, 1, &quot;Second&quot;)\n to_list(new_list2)\n // -&gt; [&quot;Test&quot;, &quot;Second&quot;]\n \n \nSynonyms:\ntreelist.to_list\ntreelist to_list","url":"treelist.html#to_list"},{"doc":"treelist","title":"try_map","content":"pub fn try_map(\n  tlist: TreeList(a),\n  filter_fn: fn(a) -&gt; Result(b, c),\n) -&gt; Result(TreeList(b), c)\n Takes a function that returns a `Result` and applies it to each element in a\n given treelist in turn.\n\n If the function returns `Ok(new_value)` for all elements in the treelist then a\n treelist of the new values is returned.\n\n If the function returns `Error(reason)` for any of the elements then it is\n returned immediately. None of the elements in the treelist are processed after\n one returns an `Error`.\n\n ## Examples\n\n \n let assert Ok(list) = from_list([1, 2, 3])\n let assert Ok(tl) = try_map(list, fn(x) { Ok(x + 2) })\n treelist.to_list(tl)\n // -&gt; [3, 4, 5]\n \n\n \n let assert Ok(list) = from_list([1, 2, 3])\n try_map(list, fn(_) { Error(0) })\n // -&gt; Error(0)\n \n\n \n let assert Ok(list) = from_list([[1], [2, 3]])\n let assert Ok(tl) = try_map(list, first)\n treelist.to_list(tl)\n // -&gt; Ok([1, 2])\n \n\n \n let assert Ok(list) = from_list([[1], [], [2]])\n try_map(list, first)\n // -&gt; Error(Nil)\n \n\nSynonyms:\ntreelist.try_map\ntreelist try_map","url":"treelist.html#try_map"},{"doc":"treelist","title":"wrap","content":"pub fn wrap(val: a) -&gt; TreeList(a)\n Returns the given item wrapped in a list.\n\n ## Examples\n\n \n wrap(1)\n |&gt; to_list\n // -&gt; [1]\n\n wrap([&quot;a&quot;, &quot;b&quot;, &quot;c&quot;])\n |&gt; to_list\n // -&gt; [[&quot;a&quot;, &quot;b&quot;, &quot;c&quot;]]\n\n wrap([[]])\n |&gt; to_list\n // -&gt; [[[]]]\n \n\n\nSynonyms:\ntreelist.wrap\ntreelist wrap","url":"treelist.html#wrap"},{"doc":"treelist","title":"treelist","content":" TreeLists are ordered sequence of elements stored in an efficient binary\n tree structure\n \n New elements can be added at any index of the structure and will\n be stored efficiently with O(log n) complexity\n \n Based on https://en.wikipedia.org/wiki/AVL_tree\n ","url":"treelist.html"}]);